from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer
from md5 import md5
from time import time
import urllib, os, select
from util import hexify, dehexify, fmttime
from thread import start_new_thread

class WebGuiRequestHandler(BaseHTTPRequestHandler):
	server_version = "qtorrent3_webgui/UNDEFINED"
	TMPL_DIR = os.path.join(os.path.split(__file__)[0], 'templates')
	
	def template(self, tmpl, vars = {}):
		body = open(os.path.join(self.TMPL_DIR, tmpl + '.html')).read()
		vars['VERSION'] = self.server.qtorrent.version
		for key, val in vars.items():
			body = body.replace('${%s}' % key, val)
			body = body.replace('%%{%s}' % key, val.replace('<', '&lt;').replace('>', '%gt;'))
			body = body.replace('&{%s}' % key, urllib.quote_plus(val))
		return body
	
	def do_GET(self):
		doc = self.send_head()
		self.wfile.write(doc)
	
	def do_POST(self):
		l = int(self.headers["Content-Length"])
		data = self.rfile.read(l).split("&")
		query = {}
		for i in data:
			s = i.split("=", 1)
			query[s[0]] = urllib.unquote_plus(s[1])
		doc = self.send_head(query)
		self.wfile.write(doc)
	
	def do_HEAD(self):
		self.send_head()
	
	def send_head(self, query = None):
		self.get_vars = {}
		ix = self.path.find('?')
		if ix > -1:
			self.path, args = self.path[:ix], self.path[ix+1:].split('&')
			for arg in args:
				key, val = (arg.split('=', 1) + [""])[:2]
				self.get_vars[key] = urllib.unquote_plus(val)
		session = None
		cookies = self.headers.get('Cookie', '')
		for cookie in cookies.split('; '):
			if not cookie:
				continue
			key, val = cookie.split('=', 1)
			if key == 'qtorrent':
				self.session = self.server.check_session(self, val)
				break
		else:
			self.session = self.server.new_session(self)
		
		doc = self.make_doc(query)
		self.send_header("Content-type", "text/html")
		self.send_header("Content-Length", len(doc))
		self.send_header("Set-Cookie", "qtorrent=%s;" % self.session['id'])
		self.end_headers()
		return doc
	
	def redirect(self, url):
		self.send_response(200)
		self.send_header("Location", url)
		return self.template('redirect', { 'URL': url })
	
	def expand_stats(self, key, s):
		hash = hexify(key)
		vars = {
			'INFOHASH': hash,
			'NAME': s[0],
			'STATUS': s[2],
			'PERCENT_DONE': "%.1f" % (s[3] * 100.0),
			'MB_TOTAL': "%.1f" % (s[1] / 1048576.0),
			'MB_DONE': "%.1f" % (s[1] * s[3] / 1048576.0),
			'PIECES_TOTAL': str(s[14]),
			'PIECES_DONE': str(s[15]),
			'DOWN_SPEED': "%.1f" % (s[5] / 1024.0),
			'DOWN_SPEED_TOP': "%.1f" % (s[7] / 1024.0),
			'DOWN_AMOUNT': "%.1f" % (s[9] / 1048576.0),
			'UP_SPEED': "%.1f" % (s[6] / 1024.0),
			'UP_SPEED_TOP': "%.1f" % (s[8] / 1024.0),
			'UP_AMOUNT': "%.1f" % (s[10] / 1048576.0),
			'NUM_SEEDS': str(s[11]),
			'NUM_PEERS': str(s[12]),
			'DIST_COPIES': str(s[13]),
		}
		if len(s[0]) > 30:
			vars['SHORT_NAME'] = s[0][:27] + '...'
		else:
			vars['SHORT_NAME'] = s[0]
		if s[9] > 0:
			vars['SHARE_RATIO'] = "%.3f" % (float(s[10]) / float(s[9]))
		else:
			vars['SHARE_RATIO'] = '-'
		
		if s[4] < 0:
			vars['ETA'] = 'unknown'
		else:
			vars['ETA'] = fmttime(s[4])
		
		if s[2] == 'stopped':
			vars['STOP_RESUME'] = 'resume'
		else:
			vars['STOP_RESUME'] = 'stop'
		
		if s[2] == 'downloading':
			if s[11] == 0:
				if s[13] == 0:
					vars['HEALTH_STYLE'] = 'poor'
				else:
					vars['HEALTH_STYLE'] = 'medium'
			else:
				vars['HEALTH_STYLE'] = 'good'
		elif s[2] == 'seeding':
			if s[13] == 0:
				vars['HEALTH_STYLE'] = 'poor'
			else:
				vars['HEALTH_STYLE'] = 'good'
		else:
			vars['HEALTH_STYLE'] = 'unknown'
		
		return vars
	
	def make_doc_index(self, query):
		self.send_response(200)
		body_down = ""
		body_seed = ""
		body_stopped = ""
		
		for key, s in self.server.statsData.items():
			vars = self.expand_stats(key, s)
			if s[2] == 'seeding':
				body_seed += self.template('index_row_seed', vars)
			elif s[2] == 'stopped':
				body_stopped += self.template('index_row_stopped', vars)
			else:
				body_down += self.template('index_row_download', vars)
		return self.template('index', { 'DOWNLOADS': body_down, 'SEEDS': body_seed, 'STOPPED': body_stopped })
	
	def make_doc_show(self, query):
		infohash = dehexify(self.get_vars['infohash'])
		if self.server.statsData.has_key(infohash):
			self.send_response(200)
			vars = self.expand_stats(infohash, self.server.statsData[infohash])
			return self.template('show', vars)
		else:
			print '404!'
			self.send_response(404)
			return self.template('404')
	
	def make_doc(self, query):
		print self.path
		if self.path == "/login":
			if query is None:
				self.send_response(200)
				return self.template('login')
			elif query.get('password', '') != self.server.password:
				self.send_response(200)
				return self.template('login_failed')
			else:
				self.session['authenticated'] = True
				return self.redirect('/')
		elif not self.session.get("authenticated", False):
			return self.redirect('/login')
		elif self.path == '/logout':
			self.session["authenticated"] = False
			return self.redirect('/login')
		elif self.path == '/':
			return self.make_doc_index(query)
		elif self.path == '/stop':
			self.server.qtorrent.stop(dehexify(self.get_vars['infohash']))
			return self.redirect('/')
		elif self.path == '/resume':
			self.server.qtorrent.resume(dehexify(self.get_vars['infohash']))
			return self.redirect('/')
		elif self.path == '/remove':
			self.server.qtorrent.remove(dehexify(self.get_vars['infohash']))
			return self.redirect('/')
		elif self.path == '/add':
			self.server.qtorrent.add(query['url'])
			return self.redirect('/')
		elif self.path == '/shutdown':
			self.server.die = True
			self.send_response(200)
			return self.template('shutdown')
		elif self.path == '/show':
			return self.make_doc_show(query)
		else:
			self.send_response(404)
			return self.template('404')
 
class WebGui(HTTPServer):
	def __init__(self, qtorrent, webpw, address):
		HTTPServer.__init__(self, address, WebGuiRequestHandler)
		self.qtorrent = qtorrent
		self.password = webpw
		self.sessions = {}
		self.statsData = {}
		self.die = False
		WebGuiRequestHandler.server_version = "qtorrent3_webgui/" + qtorrent.version
	
	def check_session(self, req, id):
		if not self.sessions.has_key(id):
			return self.new_session(req)
		session = self.sessions[id]
		if session['address'] != req.client_address[0]:
			del self.sessions[id]
			return self.new_session(req)
		now = time()
		if now > session['touched'] + 60*60:
			del self.sessions[id]
			return self.new_session(req)
		session['touched'] = now
		return session
	
	def new_session(self, req):
		session = {
			'address': req.client_address[0],
			'touched': time(),
		}
		id = md5(`session`).hexdigest()
		session['id'] = id
		self.sessions[id] = session
		return session
	
	def getFilename(self, obj):
		self.qtorrent.setAutosaveFilename(obj)
	
	def stats(self, stats, regular):
		self.statsData = stats
	
	def remove(self, hash):
		self.qtorrent.remove(hash)
	
	def stop(self, hash):
		self.qtorrent.stop(hash)
	
	def resume(self, hash):
		self.qtorrent.resume(hash)
	
	def add(self, url):
		self.qtorrent.add(url)
	
	def run(self):
		while not self.die:
			(infd, outfd, exfd) = select.select([self], [], [], 0.5)
			if infd:
				self.handle_request()
		self.qtorrent.shutdown()
	
	def shutdown(self):
		self.die = True
	
def run_webgui(qtorrent, webpw):
	gui = WebGui(qtorrent, webpw, ('', 6890))
	def run():
		start_new_thread(gui.run, ())
	return gui, run
