__all__ = ['QTorrent3']

import os, cPickle, sys, time
from QtGui import run_gui
from WebGui import run_webgui
from runner import Runner
from util import check_dir
from controlsock import ControlSocket

class QTorrent3:
	def __init__(self, version, torrents, config, nogui):
		self.gui = self.webgui = None
		self.version = version
		self.config_dir = os.path.expanduser('~/.qtorrent3')
		check_dir(self.config_dir)
		self.die = False
		
		self.torrent_targets = {}
		
		try:
			f = open(os.path.join(self.config_dir, 'targets'))
			targets = cPickle.load(f)
			self.torrent_targets = targets
		except:
			print 'Warning: couldn\'t load targets'
		
		self.config = config
		self.config['data_dir'] = self.config_dir
		
		self.runner = Runner(self, torrents)
		self.controlsocket = ControlSocket(self)
		self.controlsocket.start()
		
		web_ui = config['enable_web_interface'] and config['web_interface_password']
		if web_ui:
			self.webgui, start_webgui = run_webgui(self, config['web_interface_password'])
			start_webgui()
		
		if nogui:
			self.gui_inited()
		else:
			self.gui, start_gui = run_gui(self)
			start_gui()
		
		while not self.die:
			time.sleep(0.5)
		
		if self.gui:
			self.gui.shutdown()
		if self.webgui:
			self.webgui.shutdown()
		self.controlsocket.join()
		self.runner.shutdown()
	
	def shutdown(self):
		self.die = True
	
	def gui_inited(self):
		self.runner.scan()
	
	def add(self, path, autosave = False):
		self.cleanTargetPaths()
		self.runner.add(path, autosave = autosave)
	
	def cleanTargetPaths(self):
		to_del = []
		for key in self.torrent_targets.keys():
			if not self.runner.torrents.has_key(key):
				to_del.append(key)
		if to_del:
			for key in to_del:
				del self.torrent_targets[key]
			try:
				cPickle.dump(self.torrent_targets, open(os.path.join(self.config_dir, 'targets'), 'w'))
			except:
				print 'Warning: couldn\'t save targets file'
	
	def storeTargetPath(self, infohash, path):
		if path:
			self.torrent_targets[infohash] = path
		elif self.torrent_targets.has_key(infohash):
			del self.torrent_targets[infohash]
		else:
			return
		try:
			cPickle.dump(self.torrent_targets, open(os.path.join(self.config_dir, 'targets'), 'w'))
		except:
			print 'Warning: couldn\'t save targets file'
	
	def setAutosaveFilename(self, obj):
		path = os.path.join(os.path.expanduser(self.config['auto_save_to']), obj['name'])
		self.storeTargetPath(obj['infohash'], path)
		self.runner.setFilename(path)
	
	def getFilename(self, obj):
		if obj.get('autosave', False):
			self.setAutosaveFilename(obj)
		elif self.torrent_targets.has_key(obj['infohash']):
			self.runner.setFilename(self.torrent_targets[obj['infohash']])
		elif self.gui:
			self.gui.getFilename(obj)
		else:
			self.setAutosaveFilename(obj)
	
	def setFilename(self, obj, target):
		if target:
			self.storeTargetPath(obj['infohash'], target)
		self.runner.setFilename(target)
	
	def stats(self, stats, regular):
		if self.gui:
			self.gui.stats(stats, regular)
		if self.webgui:
			self.webgui.stats(stats, regular)
	
	def remove(self, infohash):
		self.storeTargetPath(infohash, None)
		self.runner.remove(infohash)

	def stop(self, infohash):
		self.runner.stop(infohash)

	def resume(self, infohash):
		self.runner.resume(infohash)

	def stopAt(self, infohash, stopAt, stopAtRatio):
		self.runner.stopAt(infohash, stopAt, stopAtRatio)
	
	def saveConfig(self):
		self.config.save()
		self.runner.configChanged()
